
/*****************************************************************************/
/*                                                                           */
/* Regles de substitution                                                    */
/*                                                                           */
/*****************************************************************************/

/* Traduction d'un terme en un autre terme en appliquant un ensemble de
 * regles de substitution
 * Format des regles : LHS >>> RHS [ if COND ]
 *
 * Lorsque deux regles sont appliquables, seule la premiere est appliquee !
 */

/* BIG BUG : subst([X>>>x],a,A) : boucle (c'est normal)
 * il faut ecrire : Y>>>x if Y==X
 */

:-module(subst,
	[
		subst/3,
		swap/4
	]
).

:-op(950,xfx,>>>).
:-op(949,xfx,if).

%:-[funlog].

/*
test:-
	T=(1+x+2)*(2+y+3),
	S=[
		(X+Y)*Z >>> X*Z+Y*Z,
		Z*(X+Y) >>> (X+Y)*Z,
		X+(Y+Z) >>> (X+Y)+Z,
		1*X >>> X,
		X*1 >>> X,
		X+Y >>> Z if (number(X), number(Y), Z is X+Y),
		X*Y >>> Z if (number(X), number(Y), Z is X*Y),
		X+Y >>> Y+X if (number(Y), \+ number(X)),
		X*Y >>> Y*X if (number(Y), \+ number(X))
	],
	subst(S,T,TS),
	writef("%t >>> %t\n",[T,TS]).
*/

:-module_transparent subst/3, swap/4.
% pour pouvoir tester les conditions !

subst(S,T,TS):-
	/* On decompose le terme T en foncteur F et arguments A
	 * on applique la substitution aux arguments
	 */
	T=..[F|A],
	(	A=[],
		AS=[]
	;	maplist(subst(S),A,AS)
	),
	!,
	/* Maintenant, on applique la substitution a F(AS)
	 */
	TS0=..[F|AS],
	(	member(LHS0>>>RHS0,S),
		copy_term(LHS0>>>RHS0,LHS>>>RHS),
		match(LHS,TS0),
		(	var(RHS)							% le resultat est une variable
		->	TS=RHS0								% on lie TS avec l'original !!!
		;	RHS=(TS1 if COND)
		->	COND,
			subst(S,TS1,TS)
		;	subst(S,RHS,TS)
		)
	;	TS=TS0
	),
	!.

subst(_,X,X).

/* la deuxieme version est plus rapide !
match(X,Y):-
	(	var(X)
	->	X=Y
	;	X=..[F|AX],
		Y=..[F|AY],				% rem : si var(Y) : echec (c'est ce qu'il faut)
		matchall(AX,AY)
	).
*/

match(X,Y):-
	(	X=..[F|AX]				% ok si nonvar(X)
	->	Y=..[F|AY],				% echec si var(Y) (c'est ce que l'on veut !)
		matchall(AX,AY)
	;	X=Y						% ici X est une variable !!!
	).

matchall([],[]):-!.
matchall([X|RX],[Y|RY]):-
	match(X,Y),
	matchall(RX,RY).

swap(X,Y,T,Tyx):-
	subst(
		[
			X >>> NY,
			Y >>> NX
		],
		T, Tyx
	),
	NX=X, NY=Y.

/*
compile_subst(SUBST_LIST,MODULE):-
	forall(clause(MODULE:_,_,REF),erase(REF)),
	forall(member(SUBST,SUBST_LIST),
		(	(	SUBST=(A >>> B if C)
			->	trans_def(subst(q(AA)) <- subst(q(B)) if (q(subst:match(A,AA)) and q(C)),RULE)
			;	trans_def(subst(q(AA)) <- subst(q(B)) if q(subst:match(A,AA)),RULE)
			),
			MODULE:assertz(RULE)
		)
	),
	MODULE:assertz((
		subst(T,TS):-
			T=..[F|A],
			maplist(subst,A,AS),
			TS=..[F|AS]
	)).
*/
